option explicit

' ------------------------------------------------------------------------
' This is an IEXPRESS script that installs an application
' for all users. It needs elevated privileges to do this.
'
' The subroutine RequireAdminRights takes care of this -- it determines
' if privileges are elevated, and if not, it runs this script again
' in elevated mode, and waits for it to complete.
'
' (This will only be necessary if the target computer runs Windows 7
' or Vista, and if you create the installer .exe file on Windows 7.
' If you create the installer using the version of IExpress provided
' with Windows XP or Vista, User Account Control will automatically
' prompt for elevation).
'
' This script does several other useful things, including installing
' a Start Menu icon, and adding an Add/Remove Programs entry.
'
' Feel free to modify and use it as you wish.
' Brian Knittel
' ------------------------------------------------------------------------

    const APP_NAME     = "My Demo Program"
    const COMPANY_NAME = "My Company, Inc."

    const RECREATE_SHORTCUTS = True
    const ALL_USERS_APPLICATION_DATA = &H23&
    const ALL_USERS_PROGRAMS = &H17&
    const FILE_ATTRIBUTE_READONLY = &H1&

    dim shl, wver, fso, sap, targetdir, linkpath, key

    ' --- create helper objects

    set fso = CreateObject("Scripting.FileSystemObject")
    set sap = CreateObject("Shell.Application")
    set shl = CreateObject("WScript.Shell")

    ' --- be sure we can perform the installation
    '     (Vista recognizes an iexpress wizard as an installer and elevates it
    '     automatically, but Windows 7 doesn't, so let's be sure).

    RequireAdminRights True, False

    ' From this point forward, we should have full Administrator privileges

    ' --- copy files to folder in All Users Profile

    targetdir = fso.BuildPath(sap.Namespace(ALL_USERS_APPLICATION_DATA).Self.Path, APP_NAME)

    CreateFullPath targetdir
    AddNTFSPermissions targetdir, "Users:C"			' give all users Change permission in this folder
    SetReadOnly targetdir & "\uninstall.vbs", False		' it's there already from a previous install, remove readonly attribute, so we can overwrite it
    copyfiles
    SetReadOnly targetdir & "\uninstall.vbs", True		' make the uninstall script read-only

    ' --- make folder in Start Menu

    linkpath = fso.BuildPath(sap.Namespace(ALL_USERS_PROGRAMS).Self.Path, APP_NAME)
    CreateFullPath linkpath

    ' --- make shortcuts

    shortcut program1.exe, "", "Program One"
    shortcut program2.exe, "", "Program Two"

    ' -- set up uninstall entry in the Add/Remove Programs control panel applet

    key = "HKLM\Software\Microsoft\Windows\CurrentVersion\Uninstall\" & APP_NAME & "\"
    shl.RegWrite key & "DisplayName",     APP_NAME,     "REG_SZ"
    shl.RegWrite key & "Publisher",       COMPANY_NAME, "REG_SZ"
    shl.RegWrite key & "UninstallString", "wscript.exe //nologo """ & fso.BuildPath(targetdir, "uninstall.vbs") & """", "REG_SZ"

' -- that's all, folks

' ---------------------------------------------------------------------
' copyfiles - copy all files in current folder (the installer temp folder)
' 	      to the target folder. Creates the target folder if necessary.
' ---------------------------------------------------------------------

sub copyfiles
    dim f, uname


    for each f in fso.GetFolder(".").Files
	if lcase(f.name) <> "setup_admin.vbs" then
	    fso.CopyFile f.name, fso.BuildPath(targetdir, f.name), True
	end if
    next

end sub

' ---------------------------------------------------------------------
' shortcut - Create a shortcut for one application
' ---------------------------------------------------------------------

sub shortcut (prog, args, name)
    dim linkfile, link

    linkfile = fso.BuildPath(linkpath, name & ".lnk")
    if fso.FileExists(linkfile) then            ' shortcut already exists...
	if RECREATE_SHORTCUTS then
	    fso.GetFile(linkfile).delete        ' delete existing version of this shortcut
	else
	    exit sub		                ' don't re-create
	end if
    end if

    set link = shl.CreateShortcut(linkfile)
    link.TargetPath       = fso.BuildPath(targetdir, prog)
    link.Arguments        = args
    link.WindowStyle      = 1
    link.Description      = "Runs " & name
    link.WorkingDirectory = targetdir
    link.Save
end sub

' ---------------------------------------------------------------------
' CreateFullPath - ensures that all folders in a path exist
' ---------------------------------------------------------------------

sub CreateFullPath (byval path)
   dim parent                             ' temporary variable

   path   = fso.GetAbsolutePathname(path) ' be sure path is fully qualified
   parent = fso.GetParentFolderName(path) ' get name of parent folder

   if not fso.FolderExists(parent) then   ' if necessary create parent(s)
        CreateFullPath parent 
   end if

   if not fso.FolderExists(path) then     ' if necessary create subfolder
       fso.CreateFolder(path)
   end if
end sub

' ---------------------------------------------------------------------
' SetReadOnly - make specified file readonly or not readonly
' ---------------------------------------------------------------------

sub SetReadOnly (name, make_ro)
    dim file

    if fso.FileExists(name) then
	set file = fso.GetFile(name)
	on error resume next
	if make_ro then
	    file.Attributes = file.Attributes or FILE_ATTRIBUTE_READONLY
	else
	    file.Attributes = file.Attributes and (not FILE_ATTRIBUTE_READONLY)
	end if
	on error goto 0
    end if
end sub

' ---------------------------------------------------------------------
' AddNTFSPermissions - use CACLS to grant NTFS permissions to a file or folder
' ---------------------------------------------------------------------

sub AddNTFSPermissions (name, setting)
    if fso.FileExists(name) or fso.FolderExists(name) then
	shl.Run "%COMSPEC% /c cacls.exe """ & name & """ /e /g " & setting, 2, True
    end if
end sub

' ---------------------------------------------------------------------
' RequireAdminRights - ensure that we are running with administrator privileges.
' If verify is True, we do a test to be sure we can e.g. write to \windows\system32.
'
' If selfelevate is True and we are not currently elevated, we will have
' Windows prompt for credentials and execute this script again.
'
' (Note: the initial invocation will immediately terminate if this happens, so 
' this option is not usable in a setup script called by an iexpress installer.
' Instead, call RequireAdminRights with selfelevate = False. The user will
' be told how to request elevation manually.)
' ---------------------------------------------------------------------

sub RequireAdminRights (verify, selfelevate)

    Dim wver, elevated, arg, cmd, app, how_to_elevate

    wver = shl.RegRead("HKLM\Software\Microsoft\Windows NT\CurrentVersion\CurrentVersion")
    if wver <= 4 then					' win 3, win 9x or win NT
	if MsgBox("We suspect that this program may not work on your version of Windows. Do you want to try anyway?", _
		vbYesNo, "Windows Version " & wver & "?!") <> vbYes then wscript.Quit 1
	if not verify then exit sub
	how_to_elevate = "Log on as an Administrator and run the program again."
    elseif wver < 6 then				' win 2000 or XP
	if not verify then exit sub
	how_to_elevate = "Right-click the program icon, select Run As, then select Administrator; or, log on as an Administrator and try again."
    else
	how_to_elevate = "Right-click the program icon and select Run As Administrator."
    end if						' otherwise Vista or Windows 7.

    elevated = False					' check for
    for each arg in WScript.Arguments	        	' special cmdline arg that means we self-elevated
	if arg = "--confirmed"  then elevated = True
    next

    if not elevated then elevated = AmIElevated()	' test to see if we were started with admin rights

    if elevated then exit sub				' we seem to be good to go

    if not selfelevate then				' Don't self-elevate, tell the user to do it
	MsgBox "You must run this program with Administrator permissions. " & how_to_elevate, _
		vbOKOnly, "Requires Administrator"
	WScript.Quit 1
    end if
			
    cmd = ""						' Run ourselves again with Admin/UAC prompt
    for each arg in WScript.Arguments
	cmd = cmd & " " & Chr(34) & arg & Chr(34)
    next
    cmd = "//nologo " & Chr(34) & WScript.ScriptFullName & Chr(34) & cmd & " --confirmed"

    set app = CreateObject("Shell.Application")
    app.ShellExecute "wscript.exe", cmd, "", "runas", 1

    ' there appears to be no way to reliably wait until the elevated script finishes. So just...
    WScript.Quit 0
end sub

' ---------------------------------------------------------------------
' AmIElevated - return True if we are running with elevated permissions.
' We determine this by trying to create a file in the \windows\system32
' folder. If it succeeds, we're elevated.  However, creating the file
' may succeed if file virtualization is effect; the file may be created
' inside the user's profile. If file can be created but it shows up in
' "virtpath", then we are NOT elevated.
'
' Note that on 64-bit Windows, even with elevation, if a 32-bit installer
' tries to create files in \windows\system32, the files will be redirected
' to \windows\syswow64. This is considered successful.
' ---------------------------------------------------------------------

function AmIElevated
    dim n, testname, winpath32, winpath64, virtpath32, virtpath64
    dim testfile32, testfile64, virtfile32, virtfile64, txt

    AmIElevated = False			' assume failure

    ' we will try to make a file in the \windows\system32 folder
    winpath32  = shl.Environment("Process").Item("SystemRoot") & "\system32"

    ' on 64-bit Windows, a 32-bit installer's files will be redirected
    ' to the syswow64 folder if the process is elevated
    winpath64  = shl.Environment("Process").Item("SystemRoot") & "\sysWOW64"

    ' on Vista/Win7 the file may be redirected to the user's folder
    ' \users\USERNAME\AppData\Local\VirtualStore\Windows\System32 or sysWOW64
    ' if the process is not elevated
    virtpath32 = shl.Environment("Process").Item("LOCALAPPDATA") & _
	   "\VirtualStore\" & mid(winpath32,4)	' (skip over the "X:\" part)
    virtpath64 = shl.Environment("Process").Item("LOCALAPPDATA") & _
	   "\VirtualStore\" & mid(winpath64,4)

    for n = 1 to 100			' find a filename that doesn't already exist
	testname   = "testfile" & trim(cstr(n)) & ".tmp"
	testfile32 = fso.BuildPath(winpath32,  testname)
	testfile64 = fso.BuildPath(winpath64,  testname)
	virtfile32 = fso.BuildPath(virtpath32, testname)
	virtfile64 = fso.BuildPath(virtpath64, testname)

	if not (fso.FileExists(testfile32) or fso.FileExists(testfile64) or _
		fso.FileExists(virtfile32) or fso.FileExists(virtfile64)) then
	    err.clear
	    on error resume next	' try to create the file
	    set txt = fso.OpenTextFile(testfile32, 2, True)
	    if err.number <> 0 then
		' file create failed, which means we're not elevated
	    else
		on error goto 0
		txt.writeline "You can delete this file"
		txt.Close		' it succeeded, but...
		set txt = Nothing
		' The file create succeeded, so we might be elevated. However, it's
		' possible that the file was created in the virtualization directory.
		' So, we are really elevated only if the file was really created where wanted it
		' to be created. We can't just use FileExists() on the original file, since it 
		' would see the virtualized file. So what we have to do is look into the 
		' virtualization folder. If the file is there, we are not elevated.
		AmIElevated = not (fso.FileExists(virtfile32) or fso.FileExists(virtfile64))
		fso.DeleteFile(testfile32)
	    end if
	    exit for			' whether it succeeded or not, we have our answer
	end if
    next
end function

